import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
const xAxisTickFormat = number =>
    format('.3s')(number)

const render = data => {
  
  const xValue = d => d.date;
  const xAxisLabel = 'Year';
  
  const yValue = d => d.high;
  const yAxisLabel = 'Revenue in Millions';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 120 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top-30 - margin.bottom-20;  
// Determing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
// Determing the Scale for Y axis  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
 // POsitioning the svg element   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
 // Assigning values to x axis 
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
 // Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -90)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
 // To draw the line on the curve   and adding it to svg 
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
 // To draw the area under the curve and adding it to svg 
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};
// Loading and Parsing CSV data
csv('https://gist.githubusercontent.com/rajkumarboddapati2/d9ef75de10687fcdadc0a7c2430a4dc4/raw/5e3dd993d13cd1eac5ac6f8d762546cb577875ef/gistfile1.txt')
  .then(data => {
    data.forEach(d => {
      d.high = +d.high;
//Formatting to retrieve year from the date
      d.date = d3.timeParse("%d-%m-%Y")(d.date);
    });
   const slicedData = data.slice(5000, 15000); // slice the first 100 data points
    
    render(slicedData);
  });